#lang racket

; port our data back to a Racket list ...

(define games
  '((102 51) (78 67) (53 94) (64 75) (54 71) (64 68)))

; ... and our function to a Racket function ...

(define margin-for
  (lambda (game)
    (abs (- (first game)
            (second game)))))

; ... so that we can map and apply:

(define results (map margin-for games))
(define total (apply + results))

; results doesn't add much value here...
; (apply + (map margin-for games))

(define total-margin
  (lambda (list-of-games)
    (apply +
           (map margin-for
                list-of-games))))

; --------------------------------------------------------------------
; optional easter egg
;
; Doesn't our functional solution make two passes down
; a list of length n, instead of one?  Isn't that
; inefficient.
;
; Two passes: yes.  Inefficient: Maybe, but not necessarily.
; However, if you would prefer to make only one pass, you
; can use different sort of reducer.  A more advanced
; functional programmer might use foldl:

;; (define total-margin
;;   (lambda (list-of-games)
;;     (foldl (lambda (g sum)           ;; a function that processes p
;;              (+ sum (margin-for g))) ;;   and updates the sum
;;            0                         ;; the initial value for sum
;;            list-of-games)))          ;; the list to process

; Questions?  Please ask.
